#include <sqlextract/sqlextract.h>
#include <sqlextract/util.h>
#include <sqlextract/strid.h>
#include <sqlextract/command.h>
#include <sqlextract/tables/tablespace.h>
#include <sqlextract/tables/table.h>
#include <sqlextract/tables/table_singlerow.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Parsing */
#include <sqlextract/parser.h>
#include <sqlextract/tokenize.h>


struct sqlextract_t {
	struct strid_t *si;
	struct tablespace_t *tblsp;
	struct table_t *table_singlerow;
	logfilter_error last_error;
};

struct sqlextract_t *sqlextract_create(struct strid_t *si) {
	struct sqlextract_t *env = lf_malloc(sizeof(struct sqlextract_t));
	env->si = si;
	env->tblsp = tablespace_create();
	env->table_singlerow = table_create(&table_singlerow, env->si, NULL);
	env->last_error = ERR_OK;
	return env;
}

void sqlextract_destroy(struct sqlextract_t *env) {
	if (env == NULL)
		return;
	table_destroy(env->table_singlerow);
	tablespace_destroy(env->tblsp);
	/* We don't own the strid-instance, just borrowing it. Don't deallocate */
	free(env);
}

void sqlextract_add_table(struct sqlextract_t *env, int tblname, struct table_desc_t *tdesc, void *arg) {
	tablespace_add(env->tblsp, table_create(tdesc, env->si, arg), tblname);
}

logfilter_error sqlextract_last_error(struct sqlextract_t *env) {
	return env->last_error;
}

struct command_t *sqlextract_parse(struct sqlextract_t *env, const char *command) {
	char *tokbuf;
	const char *bufptr;
	int token;
	int toklen;

	void *par;

	struct logfilterParserParams ppars;

	bufptr = command;
	par = logfilterParserAlloc(lf_malloc);

	ppars.command = NULL;
	ppars.error = ERR_OK;
	ppars.si = env->si;
	ppars.tablespace = env->tblsp;
	ppars.table_singlerow = env->table_singlerow;

	for (;;) {
		toklen = sqlite3GetToken((const unsigned char *) bufptr, &token);
		if (token == TK_ILLEGAL) {
			break;
		}
		tokbuf = strndup(bufptr, toklen);
		bufptr += toklen;
		if (token != TK_SPACE) {
			logfilterParser(par, token, tokbuf, &ppars);
		} else {
			free(tokbuf);
		}
	}
	logfilterParser(par, TK_EOF, NULL, &ppars);

	logfilterParserFree(par, free);

	env->last_error = ppars.error;
	if (ppars.error) {
		if (ppars.command)
			command_destroy(ppars.command);
		return NULL;
	}

	return ppars.command;
}
